%% Figure 1: Lifecycle profiles

close all
load results_flex_price_EA

year = 1901;
year = year-1900;
maxage = 70;
maxage = maxage - 19;
maxageh = 63;
maxageh = maxageh - 19;

for i=1:80
    eval(['prof_c(',num2str(i),')=c_',num2str(i),'(',num2str(year),');']);
    eval(['prof_a(',num2str(i),')=a_',num2str(i),'(',num2str(year),');']);
    eval(['prof_h(',num2str(i),')=h_',num2str(i),'(',num2str(year),');']);
    eval(['prof_z(',num2str(i),')=z_',num2str(i),'(',num2str(1),');']);
end

age_prof_empirical = xlsread('Age_profiles.xlsx', 'HFCF');
age_prof_nonsmooth = xlsread('Age_profiles.xlsx', 'HFCF_nonsmooth');

minage = 24;
minage = minage - 19;

x = minage:1:maxage;
xh = minage:1:maxageh;

eprof_c = age_prof_empirical(1:maxage,4)/mean(age_prof_empirical(1:maxage,4));
nprof_c = age_prof_nonsmooth(1:maxage,4)/mean(age_prof_empirical(1:maxage,4));
prof_c = prof_c(1:end-1)/mean(prof_c(1:maxage));

eprof_h = age_prof_empirical(1:maxageh,3)/mean(age_prof_empirical(1:maxageh,3));
nprof_h = age_prof_nonsmooth(1:maxageh,3)/mean(age_prof_empirical(1:maxageh,3));
prof_h = prof_h(1:maxageh)/mean(prof_h(1:maxageh));

eprof_z = age_prof_empirical(1:maxageh,6)/mean(age_prof_empirical(1:maxageh,6));
nprof_z = age_prof_nonsmooth(1:maxageh,6)/mean(age_prof_empirical(1:maxageh,6));
prof_z = prof_z(1:maxageh)/mean(prof_z(1:maxageh));

eprof_ac = (age_prof_empirical(1:maxage,5))/mean(age_prof_empirical(1:maxage,5));
nprof_ac = (age_prof_nonsmooth(1:maxage,5))/mean(age_prof_empirical(1:maxage,5));
prof_ac = (prof_a(1:end-1))/mean(prof_a(1:maxage));

eprof_ahz = (age_prof_empirical(1:maxageh,5)./age_prof_empirical(1:maxageh,2))/mean(age_prof_empirical(1:maxageh,5)./age_prof_empirical(1:maxageh,2));
prof_ahz = (prof_a(1:maxageh)./prof_z(1:maxageh))/mean(prof_a(1:maxageh)./prof_z(1:maxageh));

figure('Position',[100,100,900,450]);

subplot(2,2,1)
area(20:98,prof_c, 'FaceColor',[0.75,0.75,0.75], 'EdgeColor','none'); 
hold on
plot(20:70,eprof_c,'-k','LineWidth',3)
plot(20:70,nprof_c,'ko')
title('Consumption')
legend('Model', 'Data (smoothed)', 'Data (raw)', 'Location', 'southwest');

subplot(2,2,2)
area(20:63,prof_h, 'FaceColor',[0.75,0.75,0.75], 'EdgeColor','none'); 
hold on
plot(20:63,eprof_h,'-k','LineWidth',3)
plot(20:63,nprof_h,'ko')
title('Hours worked')
xlim([20 62])

subplot(2,2,3)
area(20:63,prof_z, 'FaceColor',[0.75,0.75,0.75], 'EdgeColor','none'); 
hold on
plot(20:63,eprof_z,'-k','LineWidth',3)
plot(20:63,nprof_z,'ko')
title('Productivity')
xlim([20 62])

subplot(2,2,4)
area(20:98,prof_ac, 'FaceColor',[0.75,0.75,0.75], 'EdgeColor','none'); 
hold on
plot(20:70,eprof_ac,'-k','LineWidth',3)
plot(20:70,nprof_ac,'ko')
title('Assets')


%% Figure 2: Population pyramids

clear; close all;
load results_flex_price_EA

pop_empirical = xlsread('Demography_EU.xlsx', 'Implied pop (T)');

figure('Position',[100,100,900,275]);

subplot(1,2,1)
year = 1995;
year = year-1900;

y_row = year-84-1;

for i=1:80
    eval(['prof_N(',num2str(i),')=N_rel_',num2str(i),'(',num2str(year),');']);
end

p(1) = area(20:99, prof_N, 'FaceColor',[0.75,0.75,0.75], 'EdgeColor','none');
hold on;
p(2) = plot(20:85, pop_empirical(y_row, 2:67)./mean(pop_empirical(y_row, 2:10)),'-k','LineWidth',3);
title('Population pyramid in 1995')

legend(p([1 2]), 'Model', 'Data', 'Location', 'northeast');


subplot(1,2,2)
year = 2016;
year = year-1900;

y_row = year-84-1;

for i=1:80
    eval(['prof_N(',num2str(i),')=N_rel_',num2str(i),'(',num2str(year),');']);
end

p(1) = area(20:99, prof_N, 'FaceColor',[0.75,0.75,0.75], 'EdgeColor','none');
hold on;
p(2) = plot(20:99, pop_empirical(y_row, 2:81)./mean(pop_empirical(y_row, 2:10)),'-k','LineWidth',3);
title('Population pyramid in 2015')


%% Figure 3: Macroeconomic effects of the demographic transition

clear;
h1=80;
h2=180;
load results_baseline_EA
x=(1900+h1:1:1900+h2);

work=0;
retired=0;
for i=1:44
    eval(['work = work + N_rel_',num2str(i),';'])
end
for i=45:80
    eval(['retired = retired + N_rel_',num2str(i),';'])
end
dep_ratio=retired./work;

figure('Position',[100,100,1000,600]);

subplot(3,3,1);
plot(x,dep_ratio(h1:h2),'k','linewidth',2);
title('Dependency ratio')
subplot(3,3,2);
plot(x,h(h1:h2)./h(h1),'k','linewidth',2);
title('Hours worked per capita')
subplot(3,3,3);
plot(x,w(h1:h2)./w(h1),'k','linewidth',2);
title('Real hourly wage')
subplot(3,3,4);
plot(x,(k(h1:h2)./h(h1:h2))./(k(h1)./h(h1)),'k','linewidth',2);
title('Capital per hour worked')
subplot(3,3,5);
plot(x,k(h1:h2)./k(h1),'k','linewidth',2);
title('Capital per capita')
subplot(3,3,6);
plot(x,a(h1:h2)./gdp(h1:h2)/(a(h1)/gdp(h1)),'k','linewidth',2);
title('Assets-to-GDP ratio')
subplot(3,3,7);
plot(x,100*r(h1:h2),'k','linewidth',2);
title('Natural real rate of interest (%)')
subplot(3,3,8);
plot(x,gdp(h1:h2)./gdp(h1),'k','linewidth',2);
title('GDP per capita')
subplot(3,3,9);
plot(x,100*(g_gdp_pot(h1:h2)-1),'k','linewidth',2);
title('Potential GDP per capita growth rate (%)')


%% Figure 4: Model predictions against the data

close all
clear all

load 'data_macro.mat'
load 'Laubach_Williams.mat' smoothed
load results_baseline_EA inv gdp g_gdp_pot h r

data_start = 1985;
data_end = 2016;
data_bounds = data_start-1970+1:data_end-1970+1;
model_bounds = data_start-1900:data_end-1900;
data_boundsLW = data_start-1970+1-14:data_end-1970+1;
LW_bounds = data_start-1970-14:data_end-1970;
model_LW_bounds = data_start-1900-14:data_end-1900;
x = data_dates(data_bounds); 
xLW = data_dates(data_boundsLW); 

smoothed = [smoothed; NaN];
g_gdp = [NaN; gdp(2:end)./gdp(1:end-1)-1];

data_dy_pc_ma8 = [movmean(data_dy_pc(1:end-1),8); NaN];
data_h_pc_ma8 = [NaN*ones(15,1); movmean(data_h_pc(16:end),8)];
data_inv_y_ma8 = [movmean(data_inv_y(1:end-1),8); NaN];


figure('Position',[100,100,900,450]);

subplot(2,2,1);
plot(x,h(model_bounds)./h(model_bounds(1)),'k','linewidth',2); hold on
plot(x,data_h_pc_ma8(data_bounds)./data_h_pc_ma8(data_bounds(1)),'--k','linewidth',2);
title('Hours worked per capita')
xlim([x(1) x(end)])

subplot(2,2,2);
plot(x,100*inv(model_bounds)./gdp(model_bounds),'k','linewidth',2); hold on
plot(x,data_inv_y_ma8(data_bounds),'--k','linewidth',2);
title('Investment share in GDP (%)')
xlim([x(1) x(end)])
ylim([20 24])

subplot(2,2,3);
plot(x,100*g_gdp(model_bounds)-100*mean(g_gdp(model_bounds)),'k','linewidth',2); hold on
plot(x,4*data_dy_pc_ma8(data_bounds)-mean(4*data_dy_pc(data_bounds)),'--k','linewidth',2);
title('GDP per capita growth rate (%)')
xlim([x(1) x(end)])

subplot(2,2,4);
plot(xLW,100*r(model_LW_bounds),'k','linewidth',2); hold on
plot(xLW,100*smoothed(LW_bounds),'--k','linewidth',2);
title('Natural real rate of interest (%)')
xlim([xLW(1) xLW(end)])


legend('Model simulations','Data / HLW estimates','Location','southwest') %,'Orientation','horizontal'


%% Figure 5: Macroeconomic effects of the demographic transition under imperfect information

clear;
h1=80;
h2=180;
load results_baseline_EA
x=(1900+h1:1:1900+h2);

gdp1 = gdp(h1);
k1 = k(h1);

figure('Position',[100,100,900,450]);

subplot(2,2,1);
plot(x,100*R_cb_ss(h1:h2)/pi_ss-100,'k-','linewidth',2); hold on
title('Perceived natural rate of interest (%)')
subplot(2,2,2);
plot(x,100*(g_gdp_pot(h1:h2)-1),'k-','linewidth',2); hold on
title('Perceived potential GDP per capita growth rate (%)')
subplot(2,2,3);
plot(x,(pi(h1:h2)-1)*100,'k-','linewidth',2); hold on
title('Inflation (%)')
subplot(2,2,4);
plot(x,100*R(h1:h2)-100,'k-','linewidth',2); hold on
title('Nominal interest rate (%)')

load results_learning_EA_lambda_008
subplot(2,2,1);
plot(x,100*R_cb_ss(h1:h2)/pi_ss-100,'k--','linewidth',2); hold on
title('Perceived natural rate of interest (%)')
subplot(2,2,2);
plot(x,100*(g_gdp_pot(h1:h2)-1),'k--','linewidth',2); hold on
title('Perceived potential GDP per capita growth rate (%)')
subplot(2,2,3);
plot(x,(pi(h1:h2)-1)*100,'k--','linewidth',2); hold on
title('Inflation (%)')
subplot(2,2,4);
plot(x,100*R(h1:h2)-100,'k--','linewidth',2); hold on
title('Nominal interest rate (%)')


legend('Baseline','Learning');


%% Figure 6: Decomposition of inflation bias under learning

clear all;

load results_learning_EA_lambda_008 pi
bias(:,1) = pi;
load results_learningNRIonly_EA_lambda_008 pi
bias(:,2) = pi;
bias = 100*(bias-1.02);
bias(:,3) = bias(:,1)-bias(:,2);
%**************************************

h1=80;
h2=180;
x=(1900+h1:1:1900+h2);

figure('Position',[100,100,900,450]);

p(1) = area(x, (bias(h1:h2,2)+min(0,bias(h1:h2,3))), 'EdgeColor',[0.75,0.75,0.75], 'FaceColor',[0.75,0.75,0.75]);
hold on;
p(2) = area(x, (bias(h1:h2,3)), 'EdgeColor',[0.25,0.25,0.25], 'FaceColor',[0.25,0.25,0.25]);
p(3) = plot(x, (bias(h1:h2,1)), 'k-', 'LineWidth', 2);

legend(p([3 1 2]), 'Inflation bias', 'NRI misperception', 'Potential growth misperception', 'Location', 'southeast');


%% Figure 7: Annual probability of hitting the ZLB

clear all
h1=80;
h2=180;
load zlb_results_baseline_EA
x = (1900+h1:1:1900+h2);

figure('Position',[100,100,900,450]);

plot(x,zlb_prob, 'k-','linewidth',2);
hold on

load zlb_results_learning_EA_lambda_008
plot(x,zlb_prob, 'k--','linewidth',2);

legend('Baseline','Learning');
legend('Location','southeast')


%% Figure 8: Hawkish monetary policy and ZLB probability

clear all
h1=80;
h2=180;
x = (1900+h1:1:1900+h2);

figure('Position',[100,100,900,450]);

load results_baseline_EA
subplot(2,2,1)
plot(x,100*pi(h1:h2)-100, 'k-','linewidth',2);
title('Inflation')
hold on
subplot(2,2,2)
plot(x,100*R(h1:h2)-100, 'k-','linewidth',2);
title('Nominal interest rate')
hold on
load zlb_results_baseline_EA
subplot(2,2,3)
plot(x,stdR, 'k-','linewidth',2);
title('Standard dev. of nominal interest rate')
hold on
subplot(2,2,4)
plot(x,zlb_prob, 'k-','linewidth',2);
title('ZLB probability')
hold on

load results_baseline_EA_hawkishMP
subplot(2,2,1)
plot(x,100*pi(h1:h2)-100, 'k--','linewidth',2);
subplot(2,2,2)
plot(x,100*R(h1:h2)-100, 'k--','linewidth',2);
load zlb_results_baseline_EA_hawkishMP
subplot(2,2,3)
plot(x,stdR, 'k--','linewidth',2);
subplot(2,2,4)
plot(x,zlb_prob, 'k--','linewidth',2);

load results_learning_EA_lambda_008
subplot(2,2,1)
plot(x,100*pi(h1:h2)-100, 'k-.','linewidth',2);
subplot(2,2,2)
plot(x,100*R(h1:h2)-100, 'k-.','linewidth',2);
load zlb_results_learning_EA_lambda_008
subplot(2,2,3)
plot(x,stdR, 'k-.','linewidth',2);
subplot(2,2,4)
plot(x,zlb_prob, 'k-.','linewidth',2);

load results_learning_EA_lambda_008_hawkishMP
subplot(2,2,1)
plot(x,100*pi(h1:h2)-100, 'k:','linewidth',2);
subplot(2,2,2)
plot(x,100*R(h1:h2)-100, 'k:','linewidth',2);
load zlb_results_learning_EA_lambda_008_hawkishMP
subplot(2,2,3)
plot(x,stdR, 'k:','linewidth',2);
legend('Baseline','Baseline, \gamma_y=0','Learning','Learning, \gamma_y=0','Location','northeast');

subplot(2,2,4)
plot(x,zlb_prob, 'k:','linewidth',2);


%% Figure 9: Demographic transition under alternative pension systems

clear;
h1=80;
h2=180;
load results_baseline_EA
xx=(1900+h1:1:1900+h2);


figure('Position',[100,100,1000,450]);

subplot(2,3,1);
plot(xx,100*r(h1:h2)-100*r(h1),'k-','linewidth',2); hold on
title('Natural real rate of interest (%)')
subplot(2,3,3);
plot(xx,gdp(h1:h2)./gdp(h1),'k-','linewidth',2); hold on
title('GDP per capita')
subplot(2,3,2);
plot(xx,k(h1:h2)./k(h1),'k-','linewidth',2); hold on
title('Capital per capita')
subplot(2,3,4);
plot(xx,100*t_p(h1:h2),'k-','linewidth',2); hold on
title('Contribution rate (%)')
subplot(2,3,5);
plot(xx,100*rep(h1:h2),'k-','linewidth',2); hold on
title('Replacement rate (%)')


load results_pensions_EA_contrib
subplot(2,3,1);
plot(xx,100*r(h1:h2)-100*r(h1),'k--','linewidth',2); hold on
title('Natural real rate of interest (%)')
subplot(2,3,3);
plot(xx,gdp(h1:h2)./gdp(h1),'k--','linewidth',2); hold on
title('GDP per capita')
subplot(2,3,2);
plot(xx,k(h1:h2)./k(h1),'k--','linewidth',2); hold on
title('Capital per capita')
subplot(2,3,4);
plot(xx,100*t_p(h1:h2),'k--','linewidth',2); hold on
title('Contribution rate (%)')
subplot(2,3,5);
plot(xx,100*rep(h1:h2),'k--','linewidth',2); hold on
title('Replacement rate (%)')

load results_pensions_EA_repl
subplot(2,3,1);
plot(xx,100*r(h1:h2)-100*r(h1),'k:','linewidth',2); hold on
title('Natural real rate of interest (%)')
xlim([xx(1) xx(end)]);
subplot(2,3,3);
plot(xx,gdp(h1:h2)./gdp(h1),'k:','linewidth',2); hold on
title('GDP per capita')
xlim([xx(1) xx(end)]);
subplot(2,3,2);
plot(xx,k(h1:h2)./k(h1),'k:','linewidth',2); hold on
title('Capital per capita')
xlim([xx(1) xx(end)]);
subplot(2,3,4);
plot(xx,100*t_p(h1:h2),'k:','linewidth',2); hold on
title('Contribution rate (%)')
xlim([xx(1) xx(end)]);

legend('Baseline', 'Constant contribution rate', 'Constant replacement rate');
legend('Location','south');

subplot(2,3,5);
plot(xx,100*rep(h1:h2),'k:','linewidth',2); hold on
title('Replacement rate (%)')
xlim([xx(1) xx(end)]);

load zlb_results_baseline_EA
subplot(2,3,6);
plot(xx,zlb_prob,'k-','linewidth',2); hold on
load zlb_results_pensions_EA_contrib
subplot(2,3,6);
plot(xx,zlb_prob,'k--','linewidth',2); hold on
load zlb_results_pensions_EA_repl
subplot(2,3,6);
plot(xx,zlb_prob,'k:','linewidth',2); hold on
title('ZLB probability (%)')
xlim([xx(1) xx(end)]);

