// Demographic transition in a New Keynesian-OLG economy

global pensions;
// pensions: 0: no pension system; 1: constant contribution, 2: constant replacement rate
@#define pensions = 0
pensions = 0;
//must be changed twice


// J: number of OLG cohorts, JR: retirement age
@#define J	= 80
@#define JR	= 45

var
// Demography (80+2 variables)
	@#for j in 1:J
		N_rel_@{j}
	@#endfor
	
	N_rel n
		
// Households (80+81+80 variables)
	@#for j in 1:J
		c_@{j}
	@#endfor
	
	@#for j in 0:J
		a_@{j}
	@#endfor
	
	@#for j in 1:J
		h_@{j}
	@#endfor
	
// Pensions (4 variables)
	pop_ret t_p rep pen
		
// Aggregation (4 variables)
	c a h beq
	
// Financial intermediary (6 variables)
	k inv q p_d R_a b
	
// Firms (12 variables)
	mc w r_k pH_tilde pH Omega_H Upsilon_H Delta_H yH zeta_piH f gdp
	
// Monetary policy (6 variables)
	R_cb R_cb_ss R g_gdp_pot r pi

// Shocks (5 variables)
	eps_i eps_z eps_tau_H eps_u eps_gdp
	
// Reporting
	feas
    gdp_obs pi_obs R_obs c_obs inv_obs;
		
varexo
// Deterministic demography

	
// Stochastic shocks
	nu_R nu_i nu_z nu_tau_H nu_u
	;

parameters

 	n_1

	@#for j in 1:J-1
		mort_@{j}
	@#endfor 


// Households
	@#for j in 1:J
		z_@{j}
	@#endfor
	
	@#for j in 1:JR-1
		phi_@{j}
	@#endfor
	
	betta vphi
	
// Pensions
	t_p_ss rep_ss
	
// Financial intermediary
	delta s_1
	
// Firms
	alffa mu_H theta_H zeta_H tau_H
	
// Monetary policy
	gamma_R gamma_pi gamma_y pi_ss lambda lambda_gdp ad
	
// Stochastic shocks
	rho_i rho_z rho_tau_H rho_u
    std_R std_i std_z std_tau_H std_u 
	
// Programming
	dt
	;
	
// Read in external data
    //For ZLB simulations
    load zlb_temp; 
    dt = dt_temp - 1900 + 1; 
    
        mort_dt	= xlsread('Demography_EU_1900.xlsx', 'Dynare_mort');
        n_1_dt	= xlsread('Demography_EU_1900.xlsx', 'Dynare_n_y');

		nri_EA	= xlsread('EA_simul.xlsx', 'nri_baseline');

	
	D = size(mort_dt, 1);

	P = size(nri_EA, 1);

	
	age_prof = xlsread('Age_profiles.xlsx', 'Age_prof');
	
	load param_phi_closed;
	
// Assign parameter values
// Households
	@#for j in 1:J
		z_@{j} = age_prof(@{j}, 3);
	@#endfor
	
	@#for j in 1:JR-1
		phi_@{j} = phi(@{j});
	@#endfor
	
	betta	= 1.001; %
	vphi	= 4; 
	
// Pensions    
	t_p_ss = 0.135;
	rep_ss = 0.61;	
	
// Financial intermediary
	delta	= 0.1;
    s_1        = 4.3907;
	
// Firms
	alffa	= 0.25;
	mu_H	= 1.04;
    theta_H	= 0.925^4;
	zeta_H	= 0;
	tau_H	= 1;
	
// Monetary policy
	gamma_pi = 1.97;
	gamma_R	= 0.41;
    gamma_y = 0.42;

	pi_ss	= 1.02;
	
	lambda = 0.08;
	lambda_gdp = lambda;

    n_1 = n_1_dt(dt);
    @#for j in 1:J-1
        mort_@{j} = mort_dt(dt, @{j});
    @#endfor

// Stochastic shocks
    std_R       = scale*0.0091;
    std_z     = scale*0.005;
    std_u     = scale*0.028766;
    std_i     = scale*0.033345;
    std_tau_H = scale*0.019212;
    rho_z     = 0.00012222;
    rho_u     = 0.75066;
    rho_i     = 0.73423;
    rho_tau_H = 0.82207;

    ad=adj;

model;
// Demography (80+2 equations)
	N_rel_1 = 1;
	
	@#for j in 2:J
		N_rel_@{j} = (1-mort_@{j-1}(-1)) * N_rel_@{j-1}(-1)/(1+n_1);
	@#endfor
	
	N_rel = (
	@#for j in 1:J
		+ N_rel_@{j}
	@#endfor
	);
	
	n	= N_rel/N_rel(-1) * (1+n_1) - 1;
	
// Households (80+81+80 equations)
	@#for j in 1:JR-1
		c_@{j} + a_@{j} = (1-t_p)*w*z_@{j}*h_@{j} + beq + R_a/pi*a_@{j-1}(-1);
	@#endfor
	@#for j in JR:J
		c_@{j} + a_@{j} = pen + beq + R_a/pi*a_@{j-1}(-1);
	@#endfor
	
	a_@{0} = 0;
	a_@{J} = 0;
	
	@#for j in 1:J-1
        exp(eps_u)/c_@{j} = betta * exp(eps_u(+1)) * (1-mort_@{j}) * 1/c_@{j+1}(+1) * R_a(+1)/pi(+1);

	@#endfor
	
	@#for j in 1:JR-1
        h_@{j} = ((1-t_p)*w*z_@{j}/phi_@{j}/c_@{j})^(1/vphi);
	@#endfor
	@#for j in JR:J
        h_@{j} = 0;
	@#endfor
	
// Pensions (4 equations)
	pop_ret = (
	@#for j in JR:J
		+ N_rel_@{j}
	@#endfor
	)/N_rel;

	@#if pensions == 1
		t_p = t_p_ss;
		rep = t_p/((1-t_p)*pop_ret);
    @#endif
	
	@#if pensions == 2
		rep	= rep_ss;
		t_p = rep*pop_ret/(1+rep*pop_ret);
    @#endif

    @#if pensions == 0
		t_p = 0;
		rep = 0;
    @#endif
	
	pen = rep*(1-t_p)*w*h;
	
// // Aggregation (4 equations)
	c	= (
	@#for j in 1:J
		+ N_rel_@{j}*c_@{j}
	@#endfor
	)/N_rel;
	
	a = (
	@#for j in 1:J
		+ N_rel_@{j}*a_@{j}
	@#endfor
	)/(N_rel(+1)*(1+n_1(+1)));
	
	h	= (
	@#for j in 1:J
		+ N_rel_@{j}*z_@{j}*h_@{j}
	@#endfor
	)/N_rel;
	
	beq	= (
	@#for j in 2:J
		+ (N_rel_@{j-1}(-1)/(1+n_1) - N_rel_@{j}) * R_a/pi * a_@{j-1}(-1)
	@#endfor
	)/N_rel;
	
// Financial intermediary (7 equations)
	(1+n(+1))*k	= (1-delta)*k(-1) + exp(eps_i)*(1-s_1/2*(1+n)^2*(inv/inv(-1)-1)^2)*inv;
	1	= exp(eps_i)*q*(1 - s_1/2*(1+n)^2*(inv/inv(-1)-1)^2 - s_1*(1+n)^2*(inv/inv(-1)-1)*inv/inv(-1)) + exp(eps_i(+1))*pi(+1)/R*(1+n(+1))^3*(q(+1)*s_1*(inv(+1)/inv-1)*(inv(+1)/inv)^2);
	
	q	= pi(+1)/R * (r_k(+1) + q(+1)*(1-delta));
	p_d	= pi(+1)/R * (1+n(+1))*(f(+1) + p_d(+1));
	
	a	= b + (q*(1-delta)*k(-1) + inv + p_d)/(1+n(+1));
	R_a/pi*a(-1) = (r_k + (1-delta)*q)*k(-1) + R(-1)/pi*b(-1) + (f + p_d);
	
	b	= 0;
	
// Firms (12 equations)
	w	= exp(eps_z)*mc*(1-alffa)*k(-1)^alffa*h^(-alffa);
	r_k	= exp(eps_z)*mc*alffa*k(-1)^(alffa-1)*h^(1-alffa);
	
	pH_tilde	= mu_H*Omega_H/Upsilon_H;
	Omega_H		= mc*pH^(mu_H/(mu_H-1))*yH						+ theta_H*pi(+1)/R*(zeta_piH(+1)/pi(+1))^(mu_H/(1-mu_H))*Omega_H(+1);
	Upsilon_H	= exp(eps_tau_H)*tau_H*pH^(mu_H/(mu_H-1))*yH	+ theta_H*pi(+1)/R*(zeta_piH(+1)/pi(+1))^(1/(1-mu_H))*Upsilon_H(+1);
	
	pH^(1/(1-mu_H))	= theta_H*(pH(-1)*zeta_piH/pi)^(1/(1-mu_H)) + (1-theta_H)*pH_tilde^(1/(1-mu_H));
	pH	= 1;
	
	Delta_H		= (1-theta_H)*(pH_tilde/pH)^(-mu_H/(mu_H-1)) + theta_H*Delta_H(-1)*(pH/pH(-1))^(mu_H/(mu_H-1))*(zeta_piH/pi)^(-mu_H/(mu_H-1));
	zeta_piH	= zeta_H*pi(-1)+ (1-zeta_H)*pi_ss;
	
	yH*Delta_H	= exp(eps_z)*k(-1)^alffa*h^(1-alffa);
	
	f	= pH*yH - w*h - r_k*k(-1);
	gdp	= yH;
	
// Monetary policy (4 equations)
	R_cb/R_cb_ss = (R_cb(-1)/R_cb_ss(-1))^gamma_R*((pi/pi_ss)^gamma_pi*((gdp/gdp(-1))/g_gdp_pot)^gamma_y)^(1-gamma_R)* exp(std_R*nu_R);
   
    R_cb_ss	= (1+steady_state(r))*pi_ss;
    g_gdp_pot = steady_state(g_gdp_pot);
	
    R = max(1, R_cb + ad);
    
// Real interest rate definition
    r	= R/pi(+1) - 1;
	
// Shocks (5 equations)
    eps_u		= rho_u*eps_u(-1) + std_u*nu_u;
    eps_i		= rho_i*eps_i(-1) + std_i*nu_i;
    eps_z		= rho_z*eps_z(-1) + std_z*nu_z;
    eps_tau_H	= rho_tau_H*eps_tau_H(-1) + std_tau_H*nu_tau_H;
	eps_gdp		= 0; 
	
// Reporting
	feas = 100*(gdp - c - inv)/gdp;
	gdp_obs = 100*log(gdp/steady_state(gdp));
    pi_obs = 100*(pi-steady_state(pi));
    R_obs = 100*(R-steady_state(R));
    c_obs = 100*log(c/steady_state(c));
    inv_obs = 100*log(inv/steady_state(inv));

end;

    steady(nocheck);

    shocks;
    var nu_R; stderr 1;
    var nu_z; stderr 1;
    var nu_u; stderr 1;
    var nu_i; stderr 1;
    var nu_tau_H; stderr 1;
    end;

rng(26);


stoch_simul(periods=10000, order=1, irf=0) gdp_obs pi_obs R_obs r R; 
