%% stephane [DOT] adjemian [AT] ens [DOT] fr

function [ys, check] = steadystate(ys, exe)
global M_ simulation pensions

%% DO NOT CHANGE THIS PART.
%%
%% Here we load the values of the deep parameters in a loop.
%%

NumberOfParameters = M_.param_nbr;                            % Number of deep parameters.
for i = 1:NumberOfParameters                                  % Loop...
  paramname = deblank(M_.param_names(i,:));                   %    Get the name of parameter i. 
  eval([ paramname ' = M_.params(' int2str(i) ');']);         %    Get the value of parameter i.
end                                                           % End of the loop.  

NumberOfEndogenousVariables = M_.endo_nbr;                    % Number of endogenous variables.
for i = 1:NumberOfEndogenousVariables                         % Loop...
  varname = deblank(M_.endo_names(i,:));                      %    Get the name of endogenous variable i.                     
  eval([ varname ' = 0;']);                					  %    Start with 0 as default value.
end                                                           % End of the loop.

check = 0;

%%
%% END OF THE FIRST MODEL INDEPENDENT BLOCK.

%% THIS BLOCK IS MODEL SPECIFIC.
%%
%% Here the user has to define the steady state.

% Demography & external data
J  = 80;
JR = 45;

% Loading demography - to be modified so that it is taken from the mod file
mort_dt	= xlsread('Demography_EU_1900.xlsx', 'Dynare_mort');
n_1_dt	= xlsread('Demography_EU_1900.xlsx', 'Dynare_n_y');
z_dt = xlsread('Age_profiles.xlsx', 'Age_prof');

mort = mort_dt(dt,:);
n_1  = n_1_dt(dt);
z    = z_dt(1:J,3);

N_rel_all = zeros(J,1);
N_rel_all(1) = 1;
for j = 2:J
    N_rel_all(j) = (1-mort(j-1)) * N_rel_all(j-1)/(1+n_1);
end
N_rel = sum(N_rel_all);
n = n_1;

%Loading phi parameters
load param_phi_closed;

%% Solving for steady state
q	= 1;
qq	= 1;
pi	= pi_ss;

pop_ret	= sum(N_rel_all(JR:J))/N_rel;
if (pensions == 0 | pensions == 1)
    t_p = t_p_ss;
elseif pensions == 2
    t_p = rep_ss*pop_ret/(1+rep_ss*pop_ret);
end


global start options
start.r   = 0.01;
start.beq = 0.01;
start.pen = 0.0;
start.c1  = 0.5;

options	= optimset('MaxIter', 10000, 'MaxFunEvals', 1000, 'TolFun', 1e-08);

fun     = @(x)solve_rbeqpen(x, alffa, betta, delta, vphi, JR, J, pi, mu_H, ...
                            mort, n, N_rel_all, N_rel, t_p_ss, rep_ss, phi, z);
[x,~]	= fsolve(fun, [start.r, start.beq, start.pen], options);

r   = x(1);
beq = x(2);
pen	= x(3);

R_a	= pi*(1+r);
R	= R_a;
r_k	= R/pi - 1 + delta;

pH	= 1;
mc	= pH/mu_H;
w	= (1-alffa)*alffa^(alffa/(1-alffa))*r_k^(-alffa/(1-alffa))*mc^(1/(1-alffa));


%% Solve the problem of OLG households given prices, taxes and transfers
% Calculate the sum of lifetime discount factors
surv = cumprod(1-mort);
disc = 1;
for j = 2:J
    disc = disc + betta^(j-1) * surv(j-1);
end

% Calculate the NPV of lifetime transfers
lti = 0;
for j = 1:JR-1
    lti = lti + beq/(1+r)^(j-1);
end
for j = JR:J
    lti = lti + (pen+beq)/(1+r)^(j-1);
end

% Use FOCs to calculate the NPV sum of lifetime labor income
cumw = ((1-t_p)*w*z(1))^(1+1/vphi) * phi(1)^(-1/vphi);
for j = 2:JR-1
    cumw = cumw + ((1-t_p)*w*z(j))^(1+1/vphi) * phi(j)^(-1/vphi) * (betta^(j-1) * (1+r)^(j-1) * surv(j-1))^(-1/vphi) / (1+r)^(j-1);
end

% Find c1 that satisfies the lifetime budget constraint
bc = @(c1) disc*c1 - cumw*c1^(-1/vphi) - lti;
[c1, ~] = fzero(bc, start.c1, options);

a_0 = 0;

c_all = zeros(J, 1);
h_all = zeros(J, 1);
a_all = zeros(J, 1);

c_all(1) = c1;
h_all(1) = ((1-t_p)*w*z(1)/phi(1)/c_all(1))^(1/vphi);
a_all(1) = (1-t_p)*w*z(1)*h_all(1) + beq + (1+r)*a_0 - c_all(1);

for j = 2:JR-1
    c_all(j) = betta*(1-mort(j-1))*c_all(j-1)*(1+r);
    h_all(j) = ((1-t_p)*w*z(j)/phi(j)/c_all(j))^(1/vphi);
    a_all(j) = (1-t_p)*w*z(j)*h_all(j) + beq + (1+r)*a_all(j-1) - c_all(j);
end

for j = JR:J
    c_all(j) = betta*(1-mort(j-1))*c_all(j-1)*(1+r);
    a_all(j) = pen + beq + (1+r)*a_all(j-1) - c_all(j);
end

c = sum(c_all.*N_rel_all)/N_rel;
h = sum(h_all.*z.*N_rel_all)/N_rel;
a = sum(a_all.*N_rel_all)/N_rel/(1+n);

% Creating output
for j = 1:J
    eval(['c_' int2str(j) ' = ' num2str(c_all(j),16) ';']);
    eval(['h_' int2str(j) ' = ' num2str(h_all(j),16) ';']);
    eval(['a_' int2str(j) ' = ' num2str(a_all(j),16) ';']);
    eval(['N_rel_' int2str(j) ' = ' num2str(N_rel_all(j),16) ';']);
end

%% Remaining variables
k	= alffa/(1-alffa)*w/r_k*h;
inv	= (delta+n)*k;
y	= c+inv;
yH	= y;
gdp	= k^alffa*h^(1-alffa);

rep = t_p/((1-t_p)*pop_ret);
crit_pen = 0;

f	= pH*yH - w*h - r_k*k;
p_d	= (1+n)*f/(r-n);

pH_tilde	= pH;
Omega_H     = mc*yH*pH^(mu_H/(mu_H-1))/(1 - pi/R*theta_H);
Upsilon_H	= tau_H*yH*pH^(mu_H/(mu_H-1))/(1 - pi/R*theta_H);
zeta_piH	= pi;
Delta_H     = 1;

R_cb	= R;
nri     = r;
R_cb_ss	= (1+nri)*pi_ss; 
g_gdp_pot = 1;

gdp_obs     = 0;
pi_obs      = 0;
R_obs       = 0;
c_obs       = 0;
inv_obs     = 0;

eps_u		= 0;
eps_i		= 0;
eps_z		= 0;
eps_tau_H	= 0;
    
%%
%% END OF THE MODEL SPECIFIC BLOCK.

%% DO NOT CHANGE THIS PART.
%%
%% Here we define the steady state values of the endogenous variables of
%% the model.
%%
NumberOfEndogenousVariables = M_.endo_nbr;                    % Number of endogenous variables.
ys = zeros(NumberOfEndogenousVariables,1);                    % Initialization of ys (steady state).
for i = 1:NumberOfEndogenousVariables                         % Loop...
  varname = deblank(M_.endo_names(i,:));                      %    Get the name of endogenous variable i.                     
  eval(['ys(' int2str(i) ') = ' varname ';']);                %    Get the steady state value of this variable.
end                                                           % End of the loop.
%%
%% END OF THE SECOND MODEL INDEPENDENT BLOCK.


%%
% For a model with quasi steady state taken from a path

% if simulation == 10
%     load quasi_ss
%     ys = zeros(M_.endo_nbr,1);                   
%     for i = 1:M_.endo_nbr                        
%       varname = deblank(M_.endo_names(i,:));   
%       idx = strmatch(varname, quasi_ss.endo_names,'exact');
%       eval(['ys(' int2str(i) ') = ' num2str(quasi_ss.endo(idx,dt),12) ';']); 
%     end    
% end


