function F = solve_rbeqpen(x, alffa, betta, delta, vphi, JR, J, pi, mu_H, ...
                           mort, n, N_rel_all, N_rel, t_p_ss, rep_ss, phi, z)
%% Initialize
global start options pensions

r   = x(1);
beq = x(2);

R_a	= pi*(1+r);
R	= R_a;
r_k	= R/pi - 1 + delta;

pH	= 1;
mc	= pH/mu_H;
w	= (1-alffa)*alffa^(alffa/(1-alffa))*r_k^(-alffa/(1-alffa))*mc^(1/(1-alffa));

% Pensions
pen	= x(3);
pop_ret	= sum(N_rel_all(JR:J))/N_rel;
if (pensions == 0 | pensions == 1)
	t_p	= t_p_ss;
elseif pensions == 2
    t_p = rep_ss*pop_ret/(1+rep_ss*pop_ret);
end
rep = t_p/((1-t_p)*pop_ret);

%% Solve the problem of OLG households given prices, taxes and transfers
% Calculate the sum of lifetime discount factors
surv = cumprod(1-mort);
disc = 1;
for j = 2:J
    disc = disc + betta^(j-1) * surv(j-1);
end

% Calculate the NPV of lifetime transfers
lti = 0;
for j = 1:JR-1
    lti = lti + beq/(1+r)^(j-1);
end
for j = JR:J
    lti = lti + (pen+beq)/(1+r)^(j-1);
end

% Use FOCs to calculate the NPV sum of lifetime labor income
cumw = ((1-t_p)*w*z(1))^(1+1/vphi) * phi(1)^(-1/vphi);
for j = 2:JR-1
    cumw = cumw + ((1-t_p)*w*z(j))^(1+1/vphi) * phi(j)^(-1/vphi) * (betta^(j-1) * (1+r)^(j-1) * surv(j-1))^(-1/vphi) / (1+r)^(j-1);
end

% Find c1 that satisfies the lifetime budget constraint
bc = @(c1) disc*c1 - cumw*c1^(-1/vphi) - lti;
[c1, ~] = fzero(bc, start.c1, options);

a_0 = 0;

c_all = zeros(J, 1);
h_all = zeros(J, 1);
a_all = zeros(J, 1);

c_all(1) = c1;
h_all(1) = ((1-t_p)*w*z(1)/phi(1)/c_all(1))^(1/vphi);
a_all(1) = (1-t_p)*w*z(1)*h_all(1) + beq + (1+r)*a_0 - c_all(1);

for j = 2:JR-1
    c_all(j) = betta*(1-mort(j-1))*c_all(j-1)*(1+r);
    h_all(j) = ((1-t_p)*w*z(j)/phi(j)/c_all(j))^(1/vphi);
    a_all(j) = (1-t_p)*w*z(j)*h_all(j) + beq + (1+r)*a_all(j-1) - c_all(j);
end

for j = JR:J
    c_all(j) = betta*(1-mort(j-1))*c_all(j-1)*(1+r);
    a_all(j) = pen + beq + (1+r)*a_all(j-1) - c_all(j);
end

c = sum(c_all.*N_rel_all)/N_rel;
h = sum(h_all.*z.*N_rel_all)/N_rel;
a = sum(a_all.*N_rel_all)/N_rel/(1+n);

%% Remaining variables
k	= alffa/(1-alffa)*w/r_k*h;
inv	= (delta+n)*k;
y	= c+inv;
yH	= y;
gdp	= k^alffa*h^(1-alffa);

crit_pen = pen - rep*(1-t_p)*w*h;


F = [gdp - yH;  
     beq - sum((N_rel_all(1:end-1)/(1+n) - N_rel_all(2:end)).*a_all(1:end-1))/N_rel*(1+r);
     crit_pen];
end

